"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
/**
 * Get Time to First Byte.
 * Learn more: https://web.dev/custom-metrics/#navigation-timing-api
 *
 * @return {Promise<number | null>}
 */
function getTimeToFirstByte() {
    return getNavigation().then(nav => (nav ? utils_1.round(nav.responseStart) : null));
}
exports.getTimeToFirstByte = getTimeToFirstByte;
/**
 * Get server timing.
 * Learn more: https://web.dev/custom-metrics/#server-timing-api
 *
 * @return {Promise<object[] | null>}
 */
function getServerTiming() {
    return getNavigation().then(nav => (nav ? nav.serverTiming || [] : null));
}
exports.getServerTiming = getServerTiming;
/**
 * Get `DOMContentLoaded` event.
 * Learn more: https://developer.mozilla.org/en-US/docs/Web/API/Window/DOMContentLoaded_event
 *
 * @param {boolean} [isAfterPromise]
 * @return {Promise<number | null>}
 */
function getDomContentLoaded(isAfterPromise = false) {
    return getNavigation().then(nav => {
        if (!nav)
            return null;
        if (nav.domContentLoadedEventEnd)
            return utils_1.round(nav.domContentLoadedEventEnd);
        if (isAfterPromise)
            return null;
        return new Promise(resolve => {
            const dclListener = () => {
                utils_1.debug('DCL listener');
                removeEventListener('DOMContentLoaded', dclListener, true);
                utils_1.raf(() => getDomContentLoaded(true).then(resolve));
            };
            addEventListener('DOMContentLoaded', dclListener, true);
        });
    });
}
exports.getDomContentLoaded = getDomContentLoaded;
/**
 * Get `loadEventEnd` event.
 * Learn more: https://developer.mozilla.org/en-US/docs/Web/API/PerformanceNavigationTiming/loadEventEnd
 *
 * @param {boolean} [isAfterPromise]
 * @return {Promise<number | null>}
 */
function getOnLoad(isAfterPromise = false) {
    return getNavigation().then(nav => {
        if (!nav)
            return null;
        if (nav.loadEventEnd)
            return utils_1.round(nav.loadEventEnd);
        if (isAfterPromise)
            return null;
        return new Promise(resolve => {
            const loadListener = () => {
                utils_1.debug('load listener');
                removeEventListener('load', loadListener, true);
                utils_1.raf(() => getOnLoad(true).then(resolve));
            };
            addEventListener('load', loadListener, true);
        });
    });
}
exports.getOnLoad = getOnLoad;
function getNavigation() {
    return new Promise(resolve => {
        if (!utils_1.perf)
            return resolve(null);
        resolve((utils_1.perf.getEntriesByType('navigation') || [null])[0]);
    });
}
exports.getNavigation = getNavigation;
