"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mitt_1 = __importDefault(require("mitt"));
const performance_observer_1 = require("./performance-observer");
const utils_1 = require("./utils");
const FCP = 'first-contentful-paint';
const FID = 'first-input-delay';
const LCP = 'largest-contentful-paint';
const CLS = 'cumulative-layout-shift';
const visibilityChange = 'visibilitychange';
/** @typedef {'first-contentful-paint' | 'first-input-delay' | 'largest-contentful-paint' | 'cumulative-layout-shift'} StrictMetricType */
/** @typedef {StrictMetricType | 'fcp' | 'lcp' | 'fid' | 'cls'} MetricType */
/** @typedef {(metric: number | null) => any} PerformanceMetricCallback */
const emitter = mitt_1.default();
/** @type {Object<string,number>} */
const values = {};
/** @type {Object<string,true>} */
const observers = {};
exports.metrics = {
    /**
     * Subscribe on the `metric`.
     *
     * @param {MetricType} metricType
     * @param {PerformanceMetricCallback} cb
     */
    on(metricType, cb) {
        const metric = normalizeMetricType(metricType);
        if (metric === FCP)
            getValueOrCreateObserver(FCP, initFcpObserver, cb);
        else if (metric === FID)
            getValueOrCreateObserver(FID, initFidObserver, cb);
        else if (metric === LCP)
            getValueOrCreateObserver(LCP, initLcpObserver, cb);
        else if (metric === CLS)
            getValueOrCreateObserver(CLS, initClsObserver, cb);
        else
            throw new Error(`Invalid metric: ${metricType}`);
        return exports.metrics;
    }
};
/** Get First Contentful Paint. Learn more: https://web.dev/fcp/ */
exports.getFirstContentfulPaint = () => getMetricValue(FCP, initFcpObserver);
/** Get First Input Delay. Learn more: https://web.dev/fid/ */
exports.getFirstInputDelay = () => getMetricValue(FID, initFidObserver);
/** Get Largest Contentful Paint. Learn more: https://web.dev/lcp/ */
exports.getLargestContentfulPaint = () => getMetricValue(LCP, initLcpObserver);
/** Get Cimmulative Layout Shift. Learn more: https://web.dev/cls/ */
exports.getCumulativeLayoutShift = () => getMetricValue(CLS, initClsObserver);
/** @param {StrictMetricType} metricName @param {Function} observer @return {Promise<number | null>} */
function getMetricValue(metricName, observer) {
    return new Promise(resolve => getValueOrCreateObserver(metricName, observer, resolve));
}
/** @param {StrictMetricType} metricName @param {Function} observer @param {PerformanceMetricCallback} cb */
function getValueOrCreateObserver(metricName, observer, cb) {
    if (values[metricName]) {
        utils_1.debug('buffered value %s=%s', metricName, values[metricName]);
        return cb(values[metricName]); // buffered by default
    }
    emitter.on(metricName, cb);
    if (!observers[metricName]) {
        utils_1.debug('init observer %s', metricName);
        observer();
        observers[metricName] = true;
    }
}
function initFcpObserver() {
    let fcpObserver = performance_observer_1.createPerformanceObserver('paint', paintEvents => {
        const fcpEvent = paintEvents.find(e => e.name === FCP);
        if (fcpEvent && fcpObserver) {
            values[FCP] = utils_1.round(fcpEvent.startTime);
            fcpObserver.disconnect();
            utils_1.debug('capture %s=%sms', FCP, values[FCP]);
            emitter.emit(FCP, values[FCP]);
        }
    });
}
function initFidObserver() {
    let fidObserver = performance_observer_1.createPerformanceObserver('first-input', ([fidEvent]) => {
        if (fidObserver) {
            values[FID] = utils_1.round(fidEvent.processingStart - fidEvent.startTime);
            fidObserver.disconnect();
            utils_1.debug('capture %s=%sms', FID, values[FID]);
            emitter.emit(FID, values[FID]);
        }
    });
}
function initLcpObserver() {
    let lcp = 0;
    let lcpObserver = performance_observer_1.createPerformanceObserver('largest-contentful-paint', lcpEvents => {
        const lastLcpEvent = lcpEvents[lcpEvents.length - 1];
        lcp = utils_1.round(lastLcpEvent.renderTime || lastLcpEvent.loadTime);
    });
    // force emit after the first interactiion
    let fidObserver = performance_observer_1.createPerformanceObserver('first-input', () => {
        if (fidObserver) {
            fidObserver.disconnect();
            utils_1.debug('force LCP after FID');
            emitLcpEvent();
        }
    });
    function emitLcpEvent() {
        if (lcpObserver) {
            lcpObserver.takeRecords();
            lcpObserver.disconnect();
            values[LCP] = lcp;
            document.removeEventListener(visibilityChange, lcpVisibilityChangeListener, true);
            utils_1.debug('capture %s=%sms', LCP, values[LCP]);
            emitter.emit(LCP, values[LCP]);
        }
    }
    function lcpVisibilityChangeListener() {
        utils_1.debug('%s %s', visibilityChange, LCP);
        emitLcpEvent();
    }
    document.addEventListener(visibilityChange, lcpVisibilityChangeListener, true);
}
function initClsObserver() {
    let cls = 0;
    let clsObserver = performance_observer_1.createPerformanceObserver('layout-shift', lsEvents => {
        lsEvents.forEach(lsEvent => {
            // Only count layout shifts without recent user input.
            // collect percentage value
            if (!lsEvent.hadRecentInput) {
                cls = utils_1.round(cls + 100 * lsEvent.value, 4);
            }
        });
    });
    function clsVisibilityChangeListener() {
        utils_1.debug('%s %s', visibilityChange, CLS);
        if (clsObserver) {
            // Force any pending records to be dispatched.
            clsObserver.takeRecords();
            clsObserver.disconnect();
            values[CLS] = cls;
            utils_1.debug('capture %s=%s%', CLS, values[CLS]);
            document.removeEventListener(visibilityChange, clsVisibilityChangeListener, true);
            emitter.emit(CLS, values[CLS]);
        }
    }
    document.addEventListener(visibilityChange, clsVisibilityChangeListener, true);
}
/** @param {MetricType} metricType @return {string} */
function normalizeMetricType(metricType) {
    const metric = metricType.toLowerCase();
    return metric === 'fcp' ? FCP : metric === 'fid' ? FID : metric === 'lcp' ? LCP : metric === 'cls' ? CLS : metric;
}
