"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
/** @typedef {{ entryType: "mark", name: string, startTime: number }} UserTimingMark */
/** @typedef {{ entryType: "measure", name: string, startTime: number, duration: number }} UserTimingMeasure */
/** @typedef {(measure: UserTimingMeasure | null) => any} UserTimingTimeEndPaintCallback */
/**
 * Create a custom performance mark with `markName` name.
 * https://developer.mozilla.org/en-US/docs/Web/API/Performance/mark
 *
 * @param {string} markName
 * @return {UserTimingMark | null}
 */
function mark(markName) {
    if (!utils_1.perf || !utils_1.perf.mark)
        return null;
    const m = utils_1.perf.mark(markName);
    return { entryType: 'mark', name: m.name, startTime: utils_1.round(m.startTime) };
}
exports.mark = mark;
/**
 * Create performance measurement `measureName` between marks, the navigation start time, or `startMarkName`.
 * https://developer.mozilla.org/en-US/docs/Web/API/Performance/measure
 *
 * @param {string} measureName
 * @param {string} [startMarkName]
 * @param {string} [endMarkName]
 * @return {UserTimingMeasure | null}
 */
function measure(measureName, startMarkName, endMarkName) {
    if (!utils_1.perf || !utils_1.perf.measure)
        return null;
    try {
        const m = utils_1.perf.measure(measureName, startMarkName, endMarkName);
        return { entryType: 'measure', name: m.name, startTime: utils_1.round(m.startTime), duration: utils_1.round(m.duration) };
    }
    catch (err) {
        console.error(err);
        return null;
    }
}
exports.measure = measure;
/**
 * Start time measurement.
 * It's similar to console.time(label).
 *
 * @param {string} label
 * @return {UserTimingMark | null}
 */
function time(label) {
    return mark(`start:${label}`);
}
exports.time = time;
/**
 * End time measurement.
 * It's similar to console.timeEnd(label).
 *
 * @param {string} label
 * @return {UserTimingMeasure | null}
 */
function timeEnd(label) {
    return measure(label, `start:${label}`);
}
exports.timeEnd = timeEnd;
/**
 * End time measurement after the last paint.
 * It's similar to console.timeEnd(label) but async.
 *
 * @param {string} label
 * @param {UserTimingTimeEndPaintCallback} [cb]
 */
function timeEndPaint(label, cb) {
    utils_1.raf(() => {
        const m = timeEnd(label);
        if (typeof cb === 'function')
            cb(m);
    });
}
exports.timeEndPaint = timeEndPaint;
